package es.uca.xmleye.gui;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.awt.Component;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import java.util.concurrent.FutureTask;
import java.util.concurrent.Callable;

/**
 * Gestiona los diálogos de notificación, pregunta y error a mostrar.
 */
public class MessageManager {
	private String tituloError;
	private String tituloPregunta;
	private Component compPadre;

	/**
	 * Crea un nuevo gestor de mensajes de usuario.
	 * 
	 * @param tituloError
	 *            Título de diálogos de error.
	 * @param tituloPregunta
	 *            Título de diálogos Sí/No.
	 * @param compPadre
	 *            Component padre de los diálogos a mostrar.
	 */
	MessageManager(String tituloError, String tituloPregunta,
			Component compPadre) {

		this.tituloError = tituloError;
		this.tituloPregunta = tituloPregunta;
		this.compPadre = compPadre;
	}

	/**
	 * Cambia el <code>Component</code> padre de los mensajes de error por
	 * defecto. Inicialmente es <code>null</code>.
	 * 
	 * @param c
	 *            Nuevo padre de todos los diálogos mostrados a partir de ahora.
	 */
	public void setCompPadre(Component c) {
		this.compPadre = c;
	}

	/**
	 * Muestra el mensaje de error <code>msg</code> en un diálogo.
	 * 
	 * @param msg
	 *            Mensaje de error.
	 */
	public void mostrarError(final String msg) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JOptionPane.showMessageDialog(compPadre, msg, tituloError,
						JOptionPane.ERROR_MESSAGE);
			}
		});
	}

	/**
	 * Muestra la notificación <code>msg</code> en un diálogo.
	 * 
	 * @param msg
	 *            Mensaje de notificación.
	 */
	public void mostrarInfo(final String msg) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JOptionPane.showMessageDialog(compPadre, msg);
			}
		});
	}

	/**
	 * Muestra un diálogo de selección Sí/No al usuario con el mensaje
	 * <code>msg</code>.
	 * 
	 * @param msg
	 *            Texto del mensaje.
	 * @return <code>true</code> si se pulsó Sí, <code>false</code> si se
	 *         pulsó No.
	 */
	public boolean preguntarSiNo(final String msg) {
		try {

			// Evitamos bloquear el propio EDT (Event Dispatching Thread)
			if (!SwingUtilities.isEventDispatchThread()) {
				FutureTask<Boolean> task = new FutureTask<Boolean>(
						new Callable<Boolean>() {
							public Boolean call() throws Exception {
								return new Boolean(
										JOptionPane.showConfirmDialog(
												compPadre, msg, tituloPregunta,
												JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION);
							}
						});

				SwingUtilities.invokeAndWait(task);
				return task.get();
			} else {
				return new Boolean(
						JOptionPane.showConfirmDialog(compPadre, msg,
								tituloPregunta, JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION);
			}
		} catch (Exception iEx) {
			return false;
		}
	}

	/**
	 * Muestra al usuario un diálogo de entrada de texto con el mensaje
	 * <code>msg</code> y el valor por defecto <code>omision</code>.
	 * 
	 * @param msg
	 *            Texto del mensaje.
	 * @param omision
	 *            Valor por defecto de la entrada.
	 * @return Entrada del usuario, o <code>null</code> si canceló la entrada.
	 */
	public String preguntarCadena(final String msg, final String omision) {
		try {

			// Evitamos bloquear el propio EDT (Event Dispatching Thread)
			if (!SwingUtilities.isEventDispatchThread()) {
				FutureTask<String> task = new FutureTask<String>(
						new Callable<String>() {
							public String call() throws Exception {
								return JOptionPane.showInputDialog(compPadre,
										msg, omision);
							}
						});

				SwingUtilities.invokeAndWait(task);
				return task.get();
			} else {
				return JOptionPane.showInputDialog(compPadre, msg, omision);
			}
		} catch (Exception iEx) {
			return null;
		}
	}
}
