package es.uca.xmleye.presentation;

/*
 Copyright (C) 2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.nio.channels.FileLock;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ExecutionException;
import java.util.Queue;
import java.util.Locale;
import java.util.ResourceBundle;

import static es.uca.xmleye.ApplicationConstants.*;
import es.uca.xmleye.Application;
import es.uca.xmleye.ParsingException;
import es.uca.xmleye.ImportException;
import es.uca.xmleye.DocumentModel;
import es.uca.xmleye.prefs.*;
import es.uca.xmleye.types.*;
import es.uca.xmleye.util.CommandBuilder;
import es.uca.xmleye.util.SimpleWrapper;
import es.uca.xmleye.util.FilterProcess;
import es.uca.xmleye.util.InputTask;
import es.uca.xmleye.util.PipeTask;
import es.uca.xmleye.xslt.*;

/**
 * Modelo de presentación de la aplicación completa.
 */
public class WndMainPresentationModel {
	private static final ResourceBundle errRes;
	private static final ResourceBundle miscRes;

	static {
		miscRes = ResourceBundle.getBundle("es.uca.xmleye.l10n.MiscMessages",
				Locale.getDefault());
		errRes = ResourceBundle.getBundle("es.uca.xmleye.l10n.ErrorMessages",
				Locale.getDefault());
	}

	private PreferencesManager gestorPrefs; // Preferencias actuales
	private StylesheetRepository repoHojas; // Hojas disponibles

	// Documentos abiertos actualmente, según fichero origen
	private Map<File, DocumentPresentationModel> docsAbiertos;

	// Tipos disponibles
	private WndTypesPresentationModel modTipos;

	private RecentDocumentHistory docsRecientes; // RecentDocumentHistory de documentos recientes
	private PreferenceBackedBoolean bActAutomatica; // ¿Actualización automática?

	private PreprocessingStylesheet hojaPreprocOmision; // Stylesheet de preprocesado por
	// omisión
	private ViewStylesheet hojaVistaOmision; // Stylesheet de visualización por
	// omisión

	private Queue<SimpleWrapper<String>> mensajesTareas; // Mensajes de estado de

	// las tareas en
	// ejecución y
	// pendientes

	/* --- CONSTRUCTORES --- */
	public WndMainPresentationModel(WndTypesPresentationModel modTipos)
			throws ParsingException, TransformationException {

		this.modTipos = modTipos;
		this.gestorPrefs = PreferencesManagerSingleton.getInstancia();
		this.docsAbiertos = new HashMap<File, DocumentPresentationModel>();
		this.repoHojas = new StylesheetRepository(StylesheetConstants.DIR_HOJAS);

		this.docsRecientes = new RecentDocumentHistory();
		this.bActAutomatica = new PreferenceBackedBoolean(gestorPrefs,
				DefaultPreferences.CLAVE_AUTOIMPORT,
				DefaultPreferences.INICIAL_AUTOIMPORT);

		this.hojaPreprocOmision = repoHojas.getHojaPreprocesado(gestorPrefs
				.getPrefEscalar(
						DefaultPreferences.CLAVE_PREPROCESADO_OMISION,
						DefaultPreferences.INICIAL_PREPROCESADO_OMISION));
		this.hojaVistaOmision = repoHojas.getHojaVisualizacion(gestorPrefs
				.getPrefEscalar(DefaultPreferences.CLAVE_VISTA_OMISION,
						DefaultPreferences.INICIAL_VISTA_OMISION));

		this.mensajesTareas = new ConcurrentLinkedQueue<SimpleWrapper<String>>();
	}

	/* ------------------------- METODOS PUBLICOS ---------------------------- */

	/* -------- Accesores para sincronización -------- */
	public List<String> getEntradasHistorial() {
		return docsRecientes.getHistorial();
	}

	public List<String> getNombresHojasPreproc() {
		return repoHojas.getNombresHojasPreprocesado();
	}

	public List<String> getNombresHojasVista() {
		return repoHojas.getNombresHojasVisualizacion();
	}

	public boolean getActualizacionAutomatica() {
		return bActAutomatica.get();
	}

	public String getTitulo(DocumentPresentationModel docActual) {
		return "XMLEye "
				+ Application.VERSION
				+ ((docActual.getNombreDocumento() != null) ? (" - " + docActual
						.getNombreDocumento())
						: "");
	}

	public boolean getOcupado() {
		return !mensajesTareas.isEmpty();
	}

	public String getMensajeEstado() {
		synchronized (mensajesTareas) {
			if (mensajesTareas.isEmpty()) {
				return miscRes.getString("status_ready");
			} else {
				return mensajesTareas.peek().get();
			}
		}
	}

	/* --- Respuesta a eventos --- */

	// / Lanzamiento de solicitud de apertura de un fichero
	public DocumentPresentationModel solicitudAbrirDocumento(File fich)
			throws ParsingException, TransformationException,
			ImportException {
		final PreprocessingStylesheet hojaPreproc;
		final ViewStylesheet hojaVista;
		final SimpleWrapper<String> elem;

		synchronized (this) {
			hojaPreproc = hojaPreprocOmision;
			hojaVista = hojaVistaOmision;
			elem = inicioTarea(miscRes.getString("status_opening"));
		}

		// Lo quitamos del historial por si hay un fallo
		docsRecientes.retirar(fich.getAbsolutePath());

		try {
			final TypeDescriptor tipo = modTipos.getTipoPorFich(fich);

			if (tipo != null && tipo.getOrdenImportacion() != null
					&& tipo.getOrdenImportacion().trim().length() > 0) {

				// Invocaremos al importador y sustituiremos fich
				File fichTmp = File.createTempFile("gen", null);
				finTarea(elem);

				FilterProcess proc = new FilterProcess(CommandBuilder
						.prepararOrden(tipo.getOrdenImportacion(),
								new String[] { CLAVE_FUENTE },
								new String[] { fich.getAbsolutePath() }), null,
						new PipeTask(fichTmp), new TareaLineasAEstado());

				int codigoProc = proc.iniciar(true);
				if (codigoProc != 0) {
					throw new ImportException(String.format(errRes
							.getString("err_importer_fail"), codigoProc));
				}

				DocumentPresentationModel res = new DocumentPresentationModel(
						new DocumentModel(fichTmp, hojaPreproc,
								fich,
								fich.getAbsoluteFile().getParent()), hojaVista,
						this);

				return res;
			} else {

				DocumentPresentationModel res = new DocumentPresentationModel(
						new DocumentModel(fich, hojaPreproc, fich, fich
								.getAbsoluteFile().getParent()), hojaVista,
						this);

				return res;
			}

		} catch (FileNotFoundException ex) {
			throw new ImportException(ex);
		} catch (IOException ex) {
			throw new ImportException(String.format(errRes
					.getString("err_import_ioex"), ex.getLocalizedMessage()),
					ex);
		} catch (InterruptedException ex) {
			throw new ImportException(ex);
		} catch (ParsingException ex) {
			throw ex;
		} catch (TransformationException ex) {
			throw ex;
		} catch (ExecutionException e) {
			throw new ImportException(e);
		} finally {
			if (elem != null) {
				finTarea(elem);
			}
		}
	}

	// / Lanzamiento de solicitud de reapertura de un fichero
	public DocumentPresentationModel solicitudReabrirDocumento(
			DocumentPresentationModel antiguo) throws ParsingException,
			TransformationException, ImportException {

		DocumentPresentationModel nuevoDoc = solicitudAbrirDocumento(antiguo
				.getFicheroFuente());
		return nuevoDoc;
	}

	// / Añadido de un documento
	public synchronized boolean agregarDocumento(
			DocumentPresentationModel nuevoDoc) throws TransformationException {

		nuevoDoc.cambioAutoImportacion(bActAutomatica.get());
		File ficheroOrigen = nuevoDoc.getFicheroOrigen();
		docsRecientes.agregar(ficheroOrigen.getAbsolutePath());		

		if (docsAbiertos.containsKey(ficheroOrigen)) {
			docsAbiertos.get(ficheroOrigen).setModeloDocumento(
					nuevoDoc.getModeloDocumento());
			nuevoDoc.cerrar();
			return false;
		} else {
			docsAbiertos.put(ficheroOrigen, nuevoDoc);
			return true;
		}
	}

	// / Cierre de un documento por su nombre
	public synchronized void cierreDocumento(File fCerrado) {
		for (File f : docsAbiertos.keySet()) {
			if (f.equals(fCerrado)) {
				cierreDocumento(docsAbiertos.get(f));
				return;
			}
		}
	}

	// / Cierre del documento actual
	public synchronized void cierreDocumento(DocumentPresentationModel docCerrado) {
		// Interrumpimos el vigilado del fichero
		docCerrado.getNotCambios().interrumpir();
		// Retiramos de la lista de abiertos
		docsAbiertos.remove(docCerrado.getFicheroOrigen());
	}

	// / Obtención de la lista de ficheros abiertos
	public List<File> getDocumentosAbiertos() {
		ArrayList<File> vFicheros = new ArrayList<File>();
		for (File f : docsAbiertos.keySet()) {
			vFicheros.add(f);
		}
		return vFicheros;
	}

	// / Obtiene la vista para un título determinado
	public DocumentPresentationModel getModeloDocumento(File fich) {
		if (docsAbiertos.containsKey(fich)) {
			return docsAbiertos.get(fich);
		}
		return null;
	}

	// / Cambio de la hoja de preprocesado por defecto para nuevos documentos
	public synchronized void cambioHojaPreprocesado(
			DocumentPresentationModel docActual, String nombreNuevaHoja)
			throws TransformationException, ParsingException {
		// Si es la misma de antes, no hacemos nada
		if (nombreNuevaHoja.equals(hojaPreprocOmision.getNombre()))
			return;

		// Si la hoja existe, cambiamos la hoja por defecto
		PreprocessingStylesheet nuevaHoja = repoHojas
				.getHojaPreprocesado(nombreNuevaHoja);
		if (nuevaHoja != null) {
			hojaPreprocOmision = nuevaHoja;
			gestorPrefs.setPrefEscalar(
					DefaultPreferences.CLAVE_PREPROCESADO_OMISION,
					nombreNuevaHoja);

			docActual.cambioHojaPreprocesado(nuevaHoja);
		}
	}

	// / Cambio de la hoja de visualización por defecto para nuevos documentos y
	// para el actual
	public synchronized void cambioHojaVisualizacion(
			DocumentPresentationModel docActual, String nombreNuevaHoja)
			throws TransformationException {
		// Si es la misma de antes, no hacemos nada
		if (nombreNuevaHoja.equals(hojaVistaOmision.getNombre()))
			return;

		// Si la hoja existe, cambiamos la hoja por defecto
		ViewStylesheet nuevaHoja = repoHojas
				.getHojaVisualizacion(nombreNuevaHoja);
		if (nuevaHoja != null) {
			hojaVistaOmision = nuevaHoja;
			gestorPrefs
					.setPrefEscalar(DefaultPreferences.CLAVE_VISTA_OMISION,
							nombreNuevaHoja);

			docActual.cambioHojaVisualizacion(nuevaHoja);
		}
	}

	// / Cambio de la autoimportación
	public synchronized void cambioAutoImportacion() {
		// Cambiamos de valor la actualizacion automatica
		final boolean nuevoValor = !bActAutomatica.get();
		fireSetAutoImportacion(nuevoValor);
	}

	private void fireSetAutoImportacion(final boolean nuevoValor) {
		bActAutomatica.set(nuevoValor);

		// Avisamos a los modelos de documento
		for (DocumentPresentationModel doc : docsAbiertos.values()) {
			doc.cambioAutoImportacion(nuevoValor);
		}
	}

	/* --- METODOS DE PAQUETE --- */
	SimpleWrapper<String> inicioTarea(String msg) {
		SimpleWrapper<String> elem = new SimpleWrapper<String>(msg);
		mensajesTareas.offer(elem);
		return elem;
	}

	String getOrdenEditor(File f) {
		return modTipos.getTipoPorFich(f).getOrdenEdicion();
	}

	void finTarea(SimpleWrapper<String> elem) {
		mensajesTareas.remove(elem);
	}

	/** CLASES INTERNAS * */

	private class TareaLineasAEstado implements InputTask {
		private InputStream is;

		public void setFlujoEntrada(InputStream is) {
			this.is = is;
		}

		public Integer call() throws Exception {
			final BufferedReader bIn = new BufferedReader(
					new InputStreamReader(is));

			int nlineas = 0;
			String linea = bIn.readLine();

			if (linea != null) {
				SimpleWrapper<String> lineaAnterior = inicioTarea(linea);

				while ((linea = bIn.readLine()) != null) {
					synchronized (mensajesTareas) {
						finTarea(lineaAnterior);
						lineaAnterior = inicioTarea(linea);
					}

					++nlineas;
				}

				finTarea(lineaAnterior);
			}

			bIn.close();
			return nlineas;
		}
	}

}
